#include <stdio.h>
#include <stdlib.h>
#include <time.h> // Pour time()

struct musique_s {
    char* titre;
    int duree; // en secondes
};

struct playlist_s {
    int nb_musiques; // nombre de musiques dans la playlist
    int capacite; // capacité de la playlist
    struct musique_s* musiques; // tableau de musiques
    int musique_courante; // indice dans la tableau de la musique en cours de lecture
};

struct playlist_s* nouvelle_playlist(int taille)
{
    struct playlist_s* p = malloc(sizeof(struct playlist_s));
    p->nb_musiques = 0;
    p->capacite = taille;
    p->musique_courante = -1;
    p->musiques = malloc(taille * sizeof(struct musique_s));

    return p;
}

int duree_playlist(struct playlist_s* playlist)
{
    int duree = 0;
    for (int i = 0; i < playlist->nb_musiques; i++)
    {
        duree += playlist->musiques[i].duree;
    }

    return duree;
}

int ajouter_musique(struct playlist_s* playlist, char* titre, int duree)
{
    if (playlist->nb_musiques == playlist->capacite)
    {
        return 0;
    }

    playlist->musiques[playlist->nb_musiques].titre = titre;
    playlist->musiques[playlist->nb_musiques].duree = duree;
    playlist->nb_musiques++;

    return 1;
}

void convertir_secondes(int total_secondes, int* minutes, int* secondes)
{
    *minutes = total_secondes / 60;
    *secondes = total_secondes % 60;
}

void afficher_playlist(struct playlist_s* playlist)
{
    for (int i = 0; i < playlist->nb_musiques; i++)
    {
        int minutes, secondes;
        convertir_secondes(playlist->musiques[i].duree, &minutes, &secondes);

        printf("%s %s (%d:%02d)\n",
            i == playlist->musique_courante ? ">" : " ",
            playlist->musiques[i].titre,
            minutes,
            secondes
        );
    }
}

struct musique_s* musique_suivante(struct playlist_s* playlist)
{
    if (playlist->musique_courante == playlist->nb_musiques-1)
    {
        return NULL;
    }

    playlist->musique_courante++;

    return &playlist->musiques[playlist->musique_courante];
}

void melanger_playlist(struct playlist_s* playlist)
{
    for (int i = playlist->nb_musiques-1; i > 0; i--)
    {
        int j = rand() % (i+1);
        struct musique_s tmp = playlist->musiques[j];
        playlist->musiques[j] = playlist->musiques[i];
        playlist->musiques[i] = tmp;
    }
}

void liberer_playlist(struct playlist_s* playlist)
{
    free(playlist->musiques);
    free(playlist);
}

int main()
{
    srand(time(NULL));

    struct playlist_s* p = nouvelle_playlist(10);

    ajouter_musique(p, "Pour que tu m'aimes encore", 251);
    ajouter_musique(p, "Let it be", 244);
    ajouter_musique(p, "Allumer le feu", 259);

    melanger_playlist(p);

    musique_suivante(p);
    musique_suivante(p);

    afficher_playlist(p);

    int minutes, secondes;
    convertir_secondes(duree_playlist(p), &minutes, &secondes);
    printf("Durée totale: %d:%d\n", minutes, secondes);

    liberer_playlist(p);


    return EXIT_SUCCESS;
}
